#
# Support for loading NIST IHEAD/MIS images in gamera
# Copyright (C) 2008-2012 Colin Baumgarten, Tobias Bolten
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from gamera.plugin import *
import _mis_support
import glob
import os

class load_mis_entry_raw(PluginFunction):
    """Load a single image entry from the given mis file as raw data"""
    category = "None"
    self_type = None
    args = Args([String("image_file_name"), Int("index of the image in file")])
    return_type = String()
    def __call__(filename, n):
        if os.path.isfile(filename):
            return _mis_support.load_mis_entry_raw(filename, n)
        else:
            raise RuntimeError("filename is not an existing regular file")
    __call__ = staticmethod(__call__)

class load_mis_raw(PluginFunction):
    """Load the complete mis file as raw data"""
    category = "None"
    self_type = None
    args = Args([String("image_file_name")])
    return_type = String()
    def __call__(filename):
        if os.path.isfile(filename):
            return _mis_support.load_mis_raw(filename)
        else:
            raise RuntimeError("filename is not an existing regular file")
    __call__ = staticmethod(__call__)


class mis_entries(PluginFunction):
    """Number of image entries in the given mis file"""
    self_type = None
    args = Args([String("image_file_name")])
    return_type = Int()
    def __call__(filename):
        if os.path.isfile(filename):
            return _mis_support.mis_entries(filename)
        else:
            raise RuntimeError("filename is not an existing regular file")
    __call__ = staticmethod(__call__)

class img_size(PluginFunction):
    """Size of each image entry in the given mis file"""
    category = "None"
    self_type = None
    args = Args([String("image_file_name")])
    return_type = Int()
    def __call__(filename):
        if os.path.isfile(filename):
            return _mis_support.img_size(filename)
        else:
            raise RuntimeError("filename is not an existing regular file")
    __call__ = staticmethod(__call__)

class load_MIS(PluginFunction):
    """Convert all images in a mis file to gamera image format and return them
as a list.

Optionally, set all images to "manually classified", if main_id is given.
"""
    category = "MIS"
    self_type = None
    args = Args([String("filename"), String("main_id", default="None")])
    return_type = ImageList("")
    def __call__(filename, main_id = None):
        if os.path.isfile(filename):
            imglist = _mis_support.load_MIS(filename,"")
        else:
            raise RuntimeError("filename is not an existing regular file")
        if main_id:
            for img in imglist:
                img.classify_manual(main_id)
        return imglist
    __call__ = staticmethod(__call__)

class MisSupportModule(PluginModule):
    category = "MIS"
    cpp_include_dirs = ["src/rnist-1.3.11/include"]
    extra_libraries = ["nist"]
    library_dirs = ["src/rnist-1.3.11/lib"]
    cpp_headers = ["mis_support.hpp"]
    functions = [load_mis_entry_raw, mis_entries, load_mis_raw,
                 img_size, load_MIS]
    author = "Colin Baumgarten, Tobias Bolten"

module = MisSupportModule()

#load_mis_entry_raw = load_mis_entry_raw()
#load_mis_raw = load_mis_raw()
mis_entries = mis_entries()
img_size = img_size()
load_MIS = load_MIS()
