/*
 *
 * Support for loading NIST IHEAD/MIS images in gamera
 * Copyright (C) 2008 Colin Baumgarten, Tobias Bolten
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef nist_support
#define nist_support

extern "C" {
	// Headers from NIST library
	#include "ihead.h"
	#include "mis.h"
	
	// redeclare functions, as in the headers parameter declaration is missing
	// and c++ doesn't like that
	MIS *readmisfile(char*);
	MIS *readmishdr(char*);
	void freemishdr(MIS*);
	void freemis(MIS*);
	unsigned char* extractmis(MIS*, int);
	int SizeFromDepth(int, int, int);
	int get_depth(IHEAD*);
}

#include <string>
#include <cstring>
#include <list>

#include "gamera.hpp"

using namespace std;

namespace Gamera {
	/** raw data size of a single image (a single mis entry) in the given mis*/
	size_t get_ImgSize(MIS *mis) {
		return SizeFromDepth(mis->entw, mis->enth, get_depth(mis->head));
	}
	
	/** raw data size of the whole mis file */
	size_t get_MisSize(MIS *mis) {
		return SizeFromDepth(mis->misw, mis->mish, get_depth(mis->head));
	}
	
	/** load raw data for a single mis entry */
	string load_mis_entry_raw(char* filename, int number) {
		MIS *mis = readmisfile(filename);
	
		unsigned char *raw = extractmis(mis, number);
		string s = string(get_ImgSize(mis), 'a');
		char *data = const_cast<char *>(s.data());
		memcpy(data, raw, get_ImgSize(mis));
	
		free(raw);
		freemis(mis);
		return s;
	}
	
	/** load raw data of a whole mis file */
	string load_mis_raw(char* filename) {
		MIS *mis = readmisfile(filename);
	
		string s = string(get_MisSize(mis), '0');
		char *data = const_cast<char *>(s.data());
		memcpy(data, mis->data, get_MisSize(mis));
	
		freemis(mis);
		return s;
	}
	
	/** raw data size of a single image (a single mis entry) in the given mis*/
	int img_size(char* filename) {
		MIS *mis = readmishdr(filename);
		int result = get_ImgSize(mis);
		freemishdr(mis);
		return result;
	}
	
	/** number of entries (images) in this mis file */
	int mis_entries(char* filename) {
		MIS *mis = readmishdr(filename);
		int result = mis->ent_num;
		freemishdr(mis);
		return result;
	}
	
/**** alternative methods, which directly create gamera images instead of 
 * reading raw data **/
	
	/* structure (bitfield) to access single bits of a byte */
	typedef struct {
		unsigned int b0 : 1;
		unsigned int b1 : 1;
		unsigned int b2 : 1;
		unsigned int b3 : 1;
		unsigned int b4 : 1;
		unsigned int b5 : 1;
		unsigned int b6 : 1;
		unsigned int b7 : 1;
	} aByte;
	
	/** set pixel to either black or white according to value */
	template<typename T> inline void setPixel(T& iter, unsigned int value) {
		static const int black = pixel_traits<OneBitPixel>::black();
		static const int white = pixel_traits<OneBitPixel>::white();
		
		if (value)
			iter.set(black);
		else
			iter.set(white);
	}
	
	/** create a gamera image from the given mis raw data */
	template<typename T> inline void load_MIS_onebit(T& image, void* data) {
		typename T::row_iterator r = image.row_begin();
		typename T::row_iterator rend = image.row_end();
		for (; r != rend; ++r) {
			typename T::col_iterator c = r.begin();
			typename T::col_iterator cend = r.end();
			for (; c != cend;) {
				setPixel(c, ((aByte*)data)->b7); c++;
				setPixel(c, ((aByte*)data)->b6); c++;
				setPixel(c, ((aByte*)data)->b5); c++;
				setPixel(c, ((aByte*)data)->b4); c++;
				setPixel(c, ((aByte*)data)->b3); c++;
				setPixel(c, ((aByte*)data)->b2); c++;
				setPixel(c, ((aByte*)data)->b1); c++;
				setPixel(c, ((aByte*)data)->b0); c++;
				// next byte
				data = (void*) ((char*)data + 1);
			}
		}
	}
	
	/** load the mis file, convert all contained images to gamera images, and 
	 * return them as a list.
     * The second argument is ignored */
	list<Image*>* load_MIS(char* filename, const char* dummy = NULL) {
		MIS *mis = readmisfile(filename);
		int entries = mis->ent_num;
		int imgSize = get_ImgSize(mis);
		int w = mis->entw, h = mis->enth;
	
		typedef TypeIdImageFactory<ONEBIT, RLE> fact;
	
		list<Image*> *result = new list<Image*>();
		void *data = mis->data;
	
		for (int i = 0; i < entries; i++) {
			fact::image_type* image = fact::create(Point(0, 0), Dim(w, h));
			load_MIS_onebit(*image, data);
			data = (void*) ((char*)data + imgSize);
			result->push_back(image);
		}
		freemis(mis);
	
		return result;
	}
}
#endif
